// TMotionBetweenFrames.cpp: implementation of the TMotionBetweenFrames class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "TMotionBetweenFrames.h"
#include "assert.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//---------------------------------------------------------------------
// Determine the camera motion between frames.    Xingquan Zhu, Last Modification, Apr.4,01
// Usages:
//
//	TMotionBetweenFrames DetectMotion1;
//	DetectMotion1.InitMotionBetweenFrames(352,288);
//	MHist=DetectMotion1.GetCameraMotion(frame1,frame2,&MotionT);

//
// Please use InitMotionBetweenFrames(w,h) to initialize the class memeber at first.
// Then, use GetCameraMotion(unsigned char *frame1, unsigned char *frame2, int *MotionType) to get the 
// camera motion type between frame "frame1" and "frame2", then, the result will be returned to "MotionType"
// and this function will return the motion histogram between these two frames, (360 in dimension).
// The definition of MotionType
//	Error	  -1
//	PAN_LEFT   1
//	PAN_RIGHT  2
//	TILT_UP    3
//	TILT_DOWN  4
//	ZOOM_IN    5
//	ZOOM_OUT   6
//	IRREGULAR  7
//	STILL      8
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

//---------------------------------------------------------------------------------------------
TMotionBetweenFrames::TMotionBetweenFrames()
{
	memset(mDirectionHist,0,360*4);
	m_iWidth=m_iHeight=0;
}

//----------------------------------------------------------------------------------------------
TMotionBetweenFrames::~TMotionBetweenFrames()
{
	delete(mFirstFrame);
	delete(mSecondFrame);

}
//-----------------------------------------------------------------------------------------------
void TMotionBetweenFrames::InitMotionBetweenFrames(int w, int h)
{
	m_iWidth=w;
	m_iHeight=h;
	mFirstFrame=new unsigned char [w*h];
	mSecondFrame=new unsigned char [w*h];
	assert(mFirstFrame!=NULL);
	assert(mSecondFrame!=NULL);
	LowPass1.InitLowPass(w,h);
	MotionVector.InitMotionVector(w,h,20,15,30,30,27,19,10);

}

//------------------------------------------------------------------------------------------
int * TMotionBetweenFrames::GetCameraMotion(unsigned char *frame1, unsigned char *frame2, int *MotionType)
{

	if((m_iWidth==0)||(m_iHeight==0))
	{
		MessageBox(NULL,"Please use InitMotionBetweenFrames to Intialize the class members at first","Error",MB_OK);
		MotionType[0]=-1;
		return(NULL);
	}
	memset(mDirectionHist,0,360*4);

	memcpy(mFirstFrame,frame1,m_iWidth*m_iHeight);
	memcpy(mSecondFrame,frame2,m_iWidth*m_iHeight);

	LowPass1.DoLowPass(mFirstFrame,mFirstFrame);
	LowPass1.DoLowPass(mSecondFrame,mSecondFrame);

	if(MotionVector.NewTwoFrameSame(mFirstFrame,mSecondFrame)==-1)
	{
		MotionType[0]=STILL;
		return(mDirectionHist);
	}

	if(MotionVector.GetCurrentMotionBase(mFirstFrame)==-1)
	{ 
		MessageBox(NULL,"Edge Overflow while determine the base region!","Error",MB_OK);
		MotionType[0]=-1;
		return(NULL);
	}
	

	if(MotionVector.CaculateWholeMotionVector(mSecondFrame)==-1)
    { 
      
		MessageBox(NULL,"Edge Overflow while Calculating the Motioin vector!","Error",MB_OK);
		MotionType[0]=-1;
		return(NULL);
    }

	MotionType[0]=MotionVector.HoughTransfer(0, mDirectionHist);
	
	return(mDirectionHist);

}

