// Video.cpp: implementation of the CVideo class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "VideoDB.h"
#include "Frame.h"
#include "Video.h"
#include "RGB2HSV.h"

#include <math.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

void init_jpeg();

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CVideo::CVideo()
{

}
//--------------------------------------------------
CVideo::~CVideo()
{
	int i;

	if (frameBuff)
		delete (frameBuff);

	for (i = 0; i < noFrame;i++)
		delete (frameSeq[i]);

	openedFile->Close();
	delete (openedFile);
}
//--------------------------------------------------

void CVideo::processVideo(CMyScrollView* downV)
{
	// some variables used for video processing between frames
	float maxHD=0;		// the max Histogram difference

	if(!initialize())
		return;

	// read the video
	int eoVideo = 0;	// the end of video flag
	while (!eoVideo)
	{
		if (!readChunck(eoVideo))
			return;	

		sprintf(downV->l,"Proccessing chunck. Frames Proccessed: %d",noReadFrame);
		downV->Invalidate();
		downV->UpdateWindow();

		processChunck(maxHD);
		storeChunckData();
	}
	sprintf(downV->l,"Make Final Processing");
	downV->Invalidate();
	downV->UpdateWindow();

	makeFinalProcessing(maxHD);

	if (g_features || g_sceneCuts || g_keyFrames || g_specialEffects)
	{
		sprintf(downV->l,"Final Analysis");
		downV->Invalidate();
		downV->UpdateWindow();

		finalAnalysis();
	}
}
//--------------------------------------------------

CVideo::CVideo(CString fileName)
{
	openedFile = new CFile(fileName,CFile::shareDenyNone);
	frameBuff = 0;
	for (int i=0; i < MCHUNCK; i++)
		frameSeq[i] = 0;
	noFrame = 0;
	noReadFrame = 0;
	sceneCutThreshold = 0;
	skippedBs = 0;
}
//--------------------------------------------------

int CVideo::readChunck(int &eof)
{
	// initialize the chunk
	if (!noReadFrame)		// this is the first chunck
	{
		frameSeq[0] = new CFrame(0,g_Image_H,g_Image_W);
		readNextFrame(frameSeq[0],eof);
		frameSeq[0]->ref = frameSeq[0];
		noReadFrame = 1;
	}
	else
	{
		/* there is a previous chunck that we need to put its last frame as 
		the first frame in the current chunck */
		delete (frameSeq[0]);
		frameSeq[0] = frameSeq[noFrame -1];
		frameSeq[0]->ref = NULL;
		for (int i=1; i < noFrame - 1; i++)
			delete (frameSeq[i]);
	}

	noFrame = 1;

	// continue reading the chunck
	while (noFrame < g_chunckSize)	// till the chunck size
	{
		frameSeq[noFrame] = new CFrame(noReadFrame, g_Image_H,g_Image_W );
		readNextFrame(frameSeq[noFrame],eof);
		frameSeq[noFrame]->id = noReadFrame;
		frameSeq[noFrame]->ref = frameSeq[noFrame - 1];
		noReadFrame++;
		noFrame++;

		if (eof)
			break;
	}
	return 1;
}
//-----------------------------------------------------------
void CVideo::processChunck(float &maxHD)
{
	for (int i=0; i < noFrame; i++) // we will scan all the frames in the chunck
	{
		// Calculate HD and get scene cuts
		if (g_sceneCuts || g_keyFrames)	// must calculate HD
		{
			if(i)	// This is NOT the first frame in the chunck
			{
				frameSeq[i]->calculateColorHD();
// Mirette needs evaluation	frameSeq[i]->calculateCHD();
				HD[frameSeq[i]->id] = frameSeq[i]->HD;
				if (frameSeq[i]->HD > maxHD)
					maxHD = frameSeq[i]->HD;
			}
			else	// this is the first frame in the chunck process it only if it's the first in the video
			{
				if (!frameSeq[i]->id) // the first frame in the video
				{
					frameSeq[i]->HD = 0;
					HD[0] = 0;
				}
			}
		}

		if (g_spatialSeg)
		{
			frameSeq[i]->spatialSegmentation();
		}

		// Calculate the temporal segmentation and the FCON threshold
		if (g_temporalSeg)
		{
			if (i)	// this is not the first frame
			{
				frameSeq[i]->temporalSegmentation();
			}
			else	// this is the fist frame in the chunck
			{
				if (!frameSeq[i]->id)	// first frame in the video
				{
					frameSeq[i]->FCONThreshold = 0;
				}
			}
		}

		if (g_MH)
			if (i)
				frameSeq[i]->motionEstimation();
	}
	
}

//----------------------------------------------------------
void CVideo::makeFinalProcessing(float maxHD)
{
	int *f, T;	// array of frequenies & threshold
	int i,j;
	char l[100];		
	// get the scenecut threshold
	if (g_sceneCuts || g_keyFrames)	// caluclate scene cuts
	{
		f = calculateFrequencies(HD,noReadFrame, 1,200,1,maxHD);
		T = getThreshold(f,201,(float)noReadFrame - 1);
		sceneCutThreshold =  (float) ((float)T/200.0)*maxHD;
		free (f);

		// if we want to write a histogram difference file to draw a graph.
		CFile *HDFile;
		HDFile= new CFile(g_metadataFile + "HD.txt",CFile::modeCreate | CFile::modeWrite);
		for (i=0; i< noReadFrame; i++)
		{
			sprintf(l,"%f\n",HD[i]);
			HDFile->Write(l,strlen(l));
		}
		// write the threshold in the file end
		sprintf(l,"%f\n",sceneCutThreshold);
		HDFile->Write(l,strlen(l));
		HDFile->Close();

		float max1,max2;
		int id1, id2;

		// store the scene cuts in the file
		// if the number of frames between scenecuts i , j are less than 30 don't 
		// we will compare the HD between frame i,i+1 and j,j+1 if diff(i) > diff(j)
		// the i is the scenecut otherwise j is the scenecut
		int prevSceneCut, crtSceneCut, tempSceneCut;
		prevSceneCut = crtSceneCut = 0;
		HD[0] = tempSceneCut = -1;
		for (i=1; i < noReadFrame;i++)
		{
			if (HD[i] > sceneCutThreshold)
			{
				if (i - prevSceneCut > 30)
					if (tempSceneCut==-1)	// no temporary scene is selected
						tempSceneCut = i;
					else
						if (i - tempSceneCut > 30)  // this frame is more than 30 frames appart from the temoprary
						{
							crtSceneCut = tempSceneCut ;
							tempSceneCut = i;
						}
						else
							if (HD[i] >= HD[tempSceneCut])
								tempSceneCut = i;
			}
			
			if (i==noReadFrame-1 && crtSceneCut != tempSceneCut)
				crtSceneCut = tempSceneCut;

			if (crtSceneCut != prevSceneCut)	// a new scene cut is selected
			{
				sprintf(l,"%d\n",crtSceneCut);
				g_outputFiles[SCFILE]->Write(l,strlen(l));

				// If key frames are requested then select the max 2 HD in the previous
				// shot to be the key frames of that shot
				if (g_keyFrames)
				{
					max1=max2 = 0;
					id1=id2 = 0;

					for (j=prevSceneCut+1; j < crtSceneCut; j++)	// move from last to current scene cut
					{
						if (HD[j] > max1)
						{
							max2= max1;
							id2 = id1;
							max1 = HD[j];
							id1 = j;
						}
						else if (HD[j] > max2)
						{
							max2 = HD[j];
							id2 = j;
						}
					}
						
					// save into skey frames files
					if (id1 > id2)
						sprintf(l,"%d\n%d\n",id2,id1);
					else
						sprintf(l,"%d\n%d\n",id1,id2);
					g_outputFiles[KFFILE]->Write(l,strlen(l));
					HD[id1] = HD[id2] = -2;
				}
				prevSceneCut = crtSceneCut;
				HD[crtSceneCut] =  -1;
			}
		}
	

		// all the scenecuts are detected but the key frames in the last
		// shot they never been detected
		if (noReadFrame - crtSceneCut > 2)	// we should detect the max 2
		{
			max1=max2 = 0;
			id1=id2 = 0;

			for (j=crtSceneCut+1; j < noReadFrame; j++)	// move from last to current scene cut
			{
				if (HD[j] > max1)
				{
					max2= max1;
					id2 = id1;
					max1 = HD[j];
					id1 = j;
				}
				else if (HD[j] > max2)
				{
					max2 = HD[j];
					id2 = j;
				}
			}
						
			// save into skey frames files
			if (id1 > id2)
				sprintf(l,"%d\n%d\n",id2,id1);
			else
				sprintf(l,"%d\n%d\n",id1,id2);
				g_outputFiles[KFFILE]->Write(l,strlen(l));
			HD[id1]=HD[id2] = -2;
		}
		else if (noReadFrame - crtSceneCut ==2)
		{
			sprintf(l,"%d\n%d\n",crtSceneCut + 1,crtSceneCut + 2);
			g_outputFiles[KFFILE]->Write(l,strlen(l));
			HD[crtSceneCut + 1] = HD[crtSceneCut + 2] = -2;
		}
		else if (noReadFrame - crtSceneCut == 1)
		{
			sprintf(l,"%d\n",crtSceneCut + 1);
			g_outputFiles[KFFILE]->Write(l,strlen(l));
			HD[crtSceneCut + 1] = -2;
		}
	}
}
//-----------------------------------------------------------
void CVideo::clearVideo()
{
	if (g_sceneCuts || g_keyFrames || g_features)
		free (HD);

	//openedAr->Close();
	if (g_sceneCuts || g_keyFrames)
	{
		g_outputFiles[SCFILE]->Close(); delete(g_outputFiles[SCFILE]);
		g_outputFiles[KFFILE]->Close(); delete(g_outputFiles[KFFILE]);
	}
	else if (g_features || g_specialEffects)
	{
		g_outputFiles[SCFILE]->Close(); delete(g_outputFiles[SCFILE]);
	}

	if (g_spatialSeg)
	{
		g_outputFiles[SSFILE]->Close(); delete(g_outputFiles[SSFILE]);
	}

	if (g_temporalSeg)
	{
		g_outputFiles[TSFILE]->Close(); delete(g_outputFiles[TSFILE]);
	}

	if (g_MH)
	{
		g_outputFiles[MVFILE]->Close(); delete(g_outputFiles[MVFILE]);
	}

	if (g_specialEffects)
	{
		g_outputFiles[SEFILE]->Close(); delete(g_outputFiles[SEFILE]);
	}

	if (g_CH)
	{
		g_outputFiles[CHFILE]->Close(); delete(g_outputFiles[CHFILE]);
		g_outputFiles[CHFFILE]->Close(); delete(g_outputFiles[CHFFILE]);
	}
	if (g_CH2)
	{
		g_outputFiles[CHHSVFILE]->Close(); delete(g_outputFiles[CHHSVFILE]);
		g_outputFiles[CHHSVFFILE]->Close(); delete(g_outputFiles[CHHSVFFILE]);
	}
	if (g_CM)
	{
		g_outputFiles[CMFILE]->Close(); delete(g_outputFiles[CMFILE]);
		g_outputFiles[CMFFILE]->Close(); delete(g_outputFiles[CMFFILE]);
	}
	if (g_CL)
	{
		g_outputFiles[CLFILE]->Close(); delete(g_outputFiles[CLFILE]);
		g_outputFiles[CLFFILE]->Close(); delete(g_outputFiles[CLFFILE]);
	}
	if (g_TT)
	{
		g_outputFiles[TTFILE]->Close(); delete(g_outputFiles[TTFILE]);
		g_outputFiles[TTFFILE]->Close(); delete(g_outputFiles[TTFFILE]);
	}
	if (g_TE)
	{
		g_outputFiles[TEFILE]->Close(); delete(g_outputFiles[TEFILE]);
		g_outputFiles[TEFFILE]->Close(); delete(g_outputFiles[TEFFILE]);
	}
	if (g_MH)
	{
		g_outputFiles[MHFILE]->Close(); delete(g_outputFiles[MHFILE]);
		g_outputFiles[MHFFILE]->Close(); delete(g_outputFiles[MHFFILE]);
	}
	if (g_DC)
	{
		g_outputFiles[DCFILE]->Close(); delete(g_outputFiles[DCFILE]);
		g_outputFiles[DCFFILE]->Close(); delete(g_outputFiles[DCFFILE]);
	}
	if (g_SC)
	{
		g_outputFiles[SCOLFILE]->Close(); delete(g_outputFiles[SCOLFILE]);
		g_outputFiles[SCOLFFILE]->Close(); delete(g_outputFiles[SCOLFFILE]);
	}
	if (g_HT)
	{
		g_outputFiles[HTFILE]->Close(); delete(g_outputFiles[HTFILE]);
		g_outputFiles[HTFFILE]->Close(); delete(g_outputFiles[HTFFILE]);
	}
	
	if (g_EH)
	{
		g_outputFiles[EHFILE]->Close(); delete(g_outputFiles[EHFILE]);
		g_outputFiles[EHFFILE]->Close(); delete(g_outputFiles[EHFFILE]);
	}
	// the data files of the key frames and video shots
	if (g_sceneCuts || g_keyFrames)
	{
		g_outputFiles[KFDATFILE]->Close(); delete(g_outputFiles[KFDATFILE]);
		g_outputFiles[VSDATFILE]->Close(); delete(g_outputFiles[VSDATFILE]);
		g_outputFiles[VSOFFDATFILE]->Close(); delete(g_outputFiles[VSOFFDATFILE]);
	}
}
//------------------------------------------------------------
int CVideo::initialize()
{
	// initialize the video because the object is created once and the processing
	// may be done several times to run different options
	
	// Rest the MPEG Decoder
	MpvDecoder.Reset();
	if (MpvDecoder.Initialize(openedFile->GetFilePath()) != MDC_SUCCESS)
	{
		AfxMessageBox("Error Initialize the MPEG decoder");
		_exit(0);
	}

	pHeaderData=MpvDecoder.GetMpvHeaders();
	if (!pHeaderData->bGOPValid)
	{
		AfxMessageBox("Error in Header files");
		return 0;
	}

	init_jpeg();
	if (!g_Image_H && !g_Image_W)
	{
 		if (MpvDecoder.DecodePicture() < 0)
 		{
 			AfxMessageBox("Can't read first frame to get frame size");
 			return 0;
 		}
		WORD OutPicWidth, OutPicHeight;
		long lItemNum=MpvDecoder.GetCurFrame(pnOutBuf,&OutPicWidth,&OutPicHeight,0);
		g_Image_H = OutPicHeight;
		g_Image_W = OutPicWidth;
	}
	motion.InitMotionBetweenFrames(g_Image_W, g_Image_H);
	
	if (frameBuff)
		delete (frameBuff);

	frameBuff = 0;

	// construct outputfile names.
	constructOutputFiles(g_metadataFile);

	// GET NUMBER OF FRAMES IN THE VIDEO
	long noOfFrames;
	if (g_sceneCuts || g_keyFrames || g_features)	// we have to calulate the scene cuts
	{
		if (!g_noFrames)		// all the frames requested
		{
			if(MpvDecoder.GetMaxFrameNum(&noOfFrames) != MDC_SUCCESS)
			{
				AfxMessageBox("Can't get number of frames in the video");
				return 0;	// fail to initialize video
			}
		}
		else
			noOfFrames = g_noFrames;

		HD = (float *) malloc(sizeof(float) * noOfFrames);		// no frames = file length /2
	}

	noReadFrame = 0;
	noFrame = 0;
	for (int i=0; i < MCHUNCK; i++)
	{
		if (frameSeq[i])
		{
			delete (frameSeq[i]);
			frameSeq[i] = 0;
		}
	}

	return 1;	// successful
}
//--------------------------------------------------------------------
void CVideo::finalAnalysis()
{
	/* to calculate the features we should process the video from the begining 
	to work on the whole video shot. Also we should process the scene cuts file
	*/

	int eoVideo, crtFrameId;
	long noOfFrames;
	char l[1000];
	unsigned char *prevY, *crtY;
	if (g_specialEffects)
	{
		prevY = (unsigned char *)malloc(sizeof(unsigned char) * g_Image_H * g_Image_W);
		crtY = (unsigned char *)malloc(sizeof(unsigned char) * g_Image_H * g_Image_W);
	}

	eoVideo = crtFrameId = g_shotStart = g_shotEnd = g_shotNo = 0;
	finalAnalysis_Init(noOfFrames);
	
	CString temp;
	CArchive SCAr(g_outputFiles[SCFILE], CArchive::load);
	CArchive *MVAr;

	if (g_MH)
		MVAr = new CArchive(g_outputFiles[MVFILE], CArchive::load);

	SCAr.ReadString(temp);	// get from scene cuts file
	g_shotEnd = atoi(temp);

	CFrame *crtFrame = new CFrame(0, g_Image_H,g_Image_W);
	CFrame *prevFrame = new CFrame(0, g_Image_H,g_Image_W);

	while(!eoVideo)
	{
		readNextFrame(crtFrame, eoVideo);
		crtFrame->id = crtFrameId;
		
		if (g_features)
			calcFeatures(crtFrame,*MVAr);
		
		
		if (g_specialEffects)
			specialEffects(crtFrame, prevFrame, crtY, prevY);

		if (crtFrameId == g_shotEnd -1 || eoVideo)	// this frame is the last frame in the video shot
		{
			if (eoVideo)
				g_shotEnd = crtFrameId+1;
			
			// write in the SQL file this shot
			if (g_sceneCuts)
			{
				char jpgFileName[300], temp2[10];
				strcpy(jpgFileName,g_metadataFile);
				sprintf(temp2,"%d.jpg",g_shotStart);
				strcat(jpgFileName,temp2);

				sprintf(l,"%d\t%d\t''\t0\t%d\t%d\t[image('format jpeg')]'%s'\n",g_videoId,g_shotNo,g_shotStart,g_shotEnd-1,jpgFileName);
				g_outputFiles[VSDATFILE]->Write(l,strlen(l));
			}

			int N = g_shotEnd - g_shotStart;
			if (g_features)
				calcFeatures_WriteShot();	// initialize after writing to files

			g_shotStart = g_shotEnd;
			SCAr.ReadString(temp);	// get from scene cuts file
			if (temp == "")	// eof
				g_shotEnd = noOfFrames;
			else
				g_shotEnd = atoi(temp);
			g_shotNo++;
		}
		crtFrameId++;
	}
	if (g_specialEffects)
	{
		specialEffects_WriteLastPart();
		free (crtY);
		free (prevY);
	}
	delete (crtFrame);
	delete (prevFrame);
}
//---------------------------------------------------------------------

void CVideo::calcFeatures_MH(CArchive & MHAr, CFrame* crtFrame)
{
	/* this assumes that it start from frame 1 not zero where nothing can
	be calulated for frame 0 */

	char l[1000],temp[30];
	if (!crtFrame->id)	// first frame
	{
		sprintf(l,"%d %d %d 0 0 0 0 0 0 0 0 0\n",g_videoId,g_shotNo,crtFrame->id);
		g_outputFiles[MHFFILE]->Write(l,strlen(l));
		return;
	}

	int i,j, pos1, pos2;
	CString tempStr,temp2Str;
	float MH[9];
	float M = g_Image_H * g_Image_W / 64;

	for (i=0; i<9; i++)
		MH[i] = 0.0;

	pos1 = pos2 = 0;
	MHAr.ReadString(tempStr);
	for (i=0; i < g_Image_H /8; i++)
		for (j=0; j < g_Image_W/8; j++)
		{
			pos1 = tempStr.Find("(",pos1+1);	
			pos2 = tempStr.Find(")",pos2+1);

			temp2Str = tempStr.Mid(pos1+1,pos2-pos1-1);	// temp2 = int,int
			crtFrame->block8[i][j]->vx = atoi(temp2Str.Left(temp2Str.Find(",")));
			crtFrame->block8[i][j]->vy = atoi(temp2Str.Mid(temp2Str.Find(",")+1));

			crtFrame->setBlockDirection(i,j);

			MH[crtFrame->block8[i][j]->direction]++;
		}

	sprintf(l,"%d %d %d",g_videoId,g_shotNo,crtFrame->id);
	for (i=0; i<9; i++)
	{
		MH[i] = MH[i] / M;
		if (!MH[i])
			sprintf(temp," %d", MH[i]);
		else
			sprintf(temp," %f", MH[i]);
		strcat(l,temp);

		sMHTotal[i] += MH[i];
	}
	strcat(l,"\n");
	g_outputFiles[MHFFILE]->Write(l,strlen(l));
}
//-------------------------------------------------------------------

void CVideo::storeChunckData()
{
	int i,j,k;
	char temp[20],l[30000];

	// store the spatial segmentation file
	if (g_spatialSeg)
	{
		for (k=0; k < noFrame; k++)
		{
			// do not consider the first frame in each chunck except the first 
			if (frameSeq[k]->id && !k)
				continue;

			sprintf(l,"%d ",frameSeq[k]->id);

			for (i=0; i<g_Image_H/4; i++)
			{
				for (j=0; j<g_Image_W/4;j++)
				{
					if (frameSeq[k]->block4[i][j]->LCON < frameSeq[k]->LCONThreshold)	// stationary
						strcat(l,"0 ");
					else		// moving
						strcat(l,"1 ");
				}
			}
			strcat(l,"\n");
			g_outputFiles[SSFILE]->Write(l,strlen(l));
		}
	}

	// store the temporal segmentation file
	if (g_temporalSeg)		// displayed in the second column , first row
	{
		// frame zero is all stationaty because it references itself
		for (k=1; k<noFrame; k++)
		{
			sprintf(l,"%d ",frameSeq[k]->id);
			for (i=0; i<g_Image_H/4; i++)
			{
				for (j=0; j<g_Image_W/4;j++)
				{
					if (frameSeq[k]->block4[i][j]->FCON < frameSeq[k]->FCONThreshold)	// stationary
						strcat(l,"0 ");
					else		// moving
						strcat(l,"1 ");
				}
			}
			strcat(l,"\n");
			g_outputFiles[TSFILE]->Write(l,strlen(l));
		}
	}

	if (g_MH)	
	{
		// should write the motion vectors to a file so it can be loaded 
		// when calculating the Motion Histogram feature.
		// start with one because frame 0 will have no effect
		for (k=1; k<noFrame; k++)
		{
			sprintf(l,"%d ",frameSeq[k]->id);
			for (int i = 0; i<g_Image_H/8; i++)
			{
				for (int j=0; j< g_Image_W/8; j++)
				{
					sprintf(temp,"(%d,%d)",frameSeq[k]->block8[i][j]->vx,frameSeq[k]->block8[i][j]->vy);
					strcat(l,temp);
				}
			}
			strcat(l,"\n");
			g_outputFiles[MVFILE]->Write(l,strlen(l));
		}
	}
}
//-------------------------------------------------------------------
void CVideo::readNextFrame(CFrame * &crtFrame, int & eoVideo)
{
	// if JPGFileName != "" the frame will be saved as JPG image on the file System
	// with the name given in the JPGFileName parameter
	int eof = 0;
	long lResult;

 	if (MpvDecoder.DecodePicture() < 0)
 	{
 		crtFrame->id = -1;
 		return;
 	}

	if (!frameBuff)	// Nothing is buffered
	{
		if (pHeaderData->PictureHeader.nPictureCodingType==I_TYPE)	// the first frame in the video
		{
			crtFrame->readFrame(openedFile, eof);
			crtFrame->id = 0;

			if (MpvDecoder.MoveToNextPicture()!=MDC_SUCCESS)
			{
				crtFrame = 0;
				return;
			}
			return;
		}

		while (pHeaderData->PictureHeader.nPictureCodingType==B_TYPE)	// skip if B
		{	// p or I copy to buffer
			skippedBs ++;
			if (MpvDecoder.MoveToNextPicture()!=MDC_SUCCESS)
			{
				crtFrame = 0;
				return;
			}
		}

		// now we got an I or P after the first I
		if (MpvDecoder.GetCurFrameNum(&lResult) != MDC_SUCCESS)
		{
			crtFrame = 0;
			return;
		}

		if (MpvDecoder.DecodePicture() < 0)
		{
			crtFrame->id = -1;
			return;
		}

		frameBuff = new CFrame(lResult - skippedBs,g_Image_H,g_Image_W);
		frameBuff->readFrame(openedFile, eof);

		if (MpvDecoder.MoveToNextPicture()!=MDC_SUCCESS)
		{
			crtFrame = 0;
			return;
		}
	}

	// get the frame number
	if (MpvDecoder.GetCurFrameNum(&lResult) != MDC_SUCCESS)
	{
		crtFrame = 0;
		return;
	}

	if (MpvDecoder.DecodePicture() < 0)
	{
		crtFrame->id = -1;
		return;
	}

	// now the frame buff has the first P after the dummy B's
	if (pHeaderData->PictureHeader.nPictureCodingType==B_TYPE)
	{
		crtFrame->id = lResult - skippedBs;
		crtFrame->readFrame(openedFile,eof);
	}
	else
	{	// P or I copy the buffer to the video and save current to buffer
		delete (crtFrame);
		crtFrame = frameBuff;
		frameBuff = new CFrame(lResult - skippedBs, g_Image_H,g_Image_W);
		frameBuff->readFrame(openedFile,eof);
	}

	if (MpvDecoder.MoveToNextPicture()!=MDC_SUCCESS || crtFrame->id + 1== g_noFrames)
		eoVideo=1;
}
//-------------------------------------------------------------------------
void CVideo::finalAnalysis_Init(long &noOfFrames)
{
	// Rest the MPEG Decoder
	MpvDecoder.Reset();
	if (MpvDecoder.Initialize(openedFile->GetFilePath()) != MDC_SUCCESS)
	{
		AfxMessageBox("Error Initialize the MPEG decoder");
		_exit(0);
	}
	pHeaderData=MpvDecoder.GetMpvHeaders();
	if (!pHeaderData->bGOPValid)
	{
		AfxMessageBox("Error in Header files");
		return ;
	}


	if (!g_noFrames)
	{
		if(MpvDecoder.GetMaxFrameNum(&noOfFrames) != MDC_SUCCESS)
		{
			AfxMessageBox("Can't get number of frames in the video");
			return;	// fail to initialize video
		}
	}
	else
		noOfFrames = g_noFrames;

	skippedBs = 0;

	if (frameBuff)
		delete (frameBuff);

	frameBuff = 0;

	// move to the start of the scenecuts file
	g_outputFiles[SCFILE]->SeekToBegin();

	// move to the begin of the motion vectors file for the Motion Histogram calculation
	if (g_MH)
		g_outputFiles[MVFILE]->SeekToBegin();

	calcFeatures_InitCounters();

	lastMotionStartFrame = lastMotionEndFrame = IRStartFrame= IREndFrame= crtMotionStartFrame= crtMotionEndFrame=0;
	crtMotionType = lastMotionType = -1;
}
//------------------------------------------------------------------
void CVideo::calcFeatures_WriteShot()
{
	char l[30000];
	int i;
	int N = g_shotEnd - g_shotStart;


	// Color Histogram
	if (g_CH)
	{
		sprintf(l,"%d %d",g_videoId,g_shotNo);

		for (i=0; i< COLQUANT; i++)
			sCHAvg[i] /= (float) N;
		strcat(l,getStringFromArray(sCHAvg,COLQUANT));
		strcat(l,getStringFromArray(sCHMin,COLQUANT));
		strcat(l,"\n");
		g_outputFiles[CHFILE]->Write(l,strlen(l));
	}

	// dominant color is working on the color histogram of the video shot
	if (g_DC)
	{
		sprintf(l,"%d %d",g_videoId,g_shotNo);		
		if (!g_CH)
			for (i=0; i< COLQUANT; i++)
				sCHAvg[i] /= (float) N;
		
		strcat(l,getDCFromCH(sCHAvg));
		strcat(l,getDCFromCH(sCHMin));
		strcat(l,"\n");
		g_outputFiles[DCFILE]->Write(l,strlen(l));
	}


	// Color Hsitogram HSV
	if (g_CH2)
	{
		sprintf(l,"%d %d",g_videoId,g_shotNo);
		for (i=0; i< HSV_DIM; i++)
			sHSVAvg[i] /= (float) N;
		
		strcat(l,getStringFromArray(sHSVAvg,HSV_DIM));
		strcat(l,getStringFromArray(sHSVMin,HSV_DIM));
		strcat(l,"\n");
		g_outputFiles[CHHSVFILE]->Write(l,strlen(l));
	}	
	
	// Color Moments
	if (g_CM)
	{
		sprintf(l,"%d %d",g_videoId,g_shotNo);
		for (i=0; i <CM_DIM; i++)
			sCMAvg[i]/=(float)N;
		strcat(l,getStringFromArray(sCMAvg,CM_DIM));
		strcat(l,"\n");
		g_outputFiles[CMFILE]->Write(l,strlen(l));
	}

	// Color Layout
	if (g_CL)
	{
		sprintf(l,"%d %d",g_videoId,g_shotNo);
		for (i=0; i < CL_DIM; i++)
			sCLAvg[i] /= (float) N;
		
		strcat(l,getStringFromArray(sCLAvg,CL_DIM));
		strcat(l,getStringFromArray(sCLMin,CL_DIM));
		strcat(l,"\n");
		g_outputFiles[CLFILE]->Write(l,strlen(l));
	}

	// Texture Tamura
	if (g_TT)
	{
		sprintf(l,"%d %d",g_videoId,g_shotNo);
		for (i=0; i< TT_DIM; i++)
			sTTAvg[i] /= (float)N;

		strcat(l,getStringFromArray(sTTAvg,TT_DIM));
		strcat(l,"\n");
		g_outputFiles[TTFILE]->Write(l,strlen(l));
	}

	// Texture edges
	if (g_TE)
	{
		sprintf(l,"%d %d",g_videoId,g_shotNo);
		for (i=0; i< TE_DIM; i++)
			sTEAvg[i]/= (float)N;

		strcat(l,getStringFromArray(sTEAvg,TE_DIM));
		strcat(l,getStringFromArray(sTEMin,TE_DIM));
		strcat(l,"\n");

		g_outputFiles[TEFILE]->Write(l,strlen(l));	
	}

	// Motion Histogram
	if (g_MH)
	{
		sprintf(l,"%d %d",g_videoId,g_shotNo);
		for (i=0; i< 9; i++)
			sMHTotal[i]/=(float) N;
		strcat(l,getStringFromArray(sMHTotal,9));
		strcat(l,"\n");
		g_outputFiles[MHFILE]->Write(l,strlen(l));	
	}

	// scalable color
	if (g_SC)
	{
		sprintf(l,"%d %d",g_videoId,g_shotNo);
		for (i=0; i< COLQUANT; i++)
			sSCAvg[i]/=(float) N;
		strcat(l,getStringFromArray(sSCAvg,COLQUANT));
		strcat(l,"\n");
		g_outputFiles[SCOLFILE]->Write(l,strlen(l));	
	}

	// Homogeneous Texture
	if (g_HT)
	{
		sprintf(l,"%d %d",g_videoId,g_shotNo);
		for (i=0; i< HT_DIM; i++)
			sHTAvg[i]/=(float) N;
		strcat(l,getStringFromArray(sHTAvg,HT_DIM));
		strcat(l,"\n");
		g_outputFiles[HTFILE]->Write(l,strlen(l));	
	}

	// Edge Histogram
	if (g_EH)
	{
		sprintf(l,"%d %d",g_videoId,g_shotNo);
		for (i=0; i<EH_DIM ; i++)
			sEHAvg[i]/=(float) N;
		strcat(l,getStringFromArray(sEHAvg,EH_DIM));
		strcat(l,"\n");
		g_outputFiles[EHFILE]->Write(l,strlen(l));	
	}

	// initialize the counters for the next shot
	calcFeatures_InitCounters();
}
//--------------------------------------------------------------
void CVideo::calcFeatures(CFrame *crtFrame, CArchive &MVAr)
{
	char l[10000], temp[50], lineId[20];
	float h[CL_DIM];

	sprintf(lineId,"%d %d %d",g_videoId,g_shotNo, crtFrame->getFrameId());
	if (g_CH || g_DC)
	{
		strcpy(l,lineId);
		strcat(l,crtFrame->getCH(h));
		if (g_CH)
			g_outputFiles[CHFFILE]->Write(l,strlen(l));
		updateShotStatistics(sCHAvg,sCHMin,h,COLQUANT);
	}

	if (g_CH2)
	{
		strcpy(l,lineId);
		strcat(l,crtFrame->getCHHSV(h));
		g_outputFiles[CHHSVFFILE]->Write(l,strlen(l));
		updateShotStatistics(sHSVAvg,sHSVMin,h,HSV_DIM);
	}
	
	if (g_DC)
	{
		strcpy(l,lineId);
		strcat(l,crtFrame->getDC());
		g_outputFiles[DCFFILE]->Write(l,strlen(l));
	}

	if (g_CM)
	{
		strcpy(l,lineId);
		strcat(l,crtFrame->getCM(h));
		g_outputFiles[CMFFILE]->Write(l,strlen(l));
		updateShotStatistics(sCMAvg,h,CM_DIM);
	}

	if (g_CL)
	{
		strcpy(l,lineId);
		strcat(l,crtFrame->getCL(h));
		g_outputFiles[CLFFILE]->Write(l,strlen(l));
		updateShotStatistics(sCLAvg,sCLMin,h,CL_DIM);
	}

	if (g_TT)
	{
		strcpy(l,lineId);
		strcat(l,crtFrame->getTT(h));
		g_outputFiles[TTFFILE]->Write(l,strlen(l));
		updateShotStatistics(sTTAvg,h,TT_DIM);
	}

	if (g_TE)
	{
		strcpy(l,lineId);
		strcat(l,crtFrame->getTE(h));
		g_outputFiles[TEFFILE]->Write(l,strlen(l));
		updateShotStatistics(sTEAvg,sTEMin,h,TE_DIM);
	}

	if (g_SC)
	{
		strcpy(l,lineId);
		strcat(l,crtFrame->getSCol(h));
		g_outputFiles[SCOLFFILE]->Write(l,strlen(l));
		updateShotStatistics(sSCAvg,h,COLQUANT);
	}

	if (g_HT)
	{
		strcpy(l,lineId);
		strcat(l,crtFrame->getHT(h));
		g_outputFiles[HTFFILE]->Write(l,strlen(l));
		updateShotStatistics(sHTAvg,h,HT_DIM);
	}

	if (g_EH)
	{
		strcpy(l,lineId);
		strcat(l,crtFrame->getEH(h));
		g_outputFiles[EHFFILE]->Write(l,strlen(l));
		updateShotStatistics(sEHAvg,h,EH_DIM);
	}

	if (g_MH)
		calcFeatures_MH(MVAr, crtFrame);

}
//----------------------------------------------------------------

void CVideo::specialEffects(CFrame *&crtFrame, CFrame *&prevFrame, unsigned char *crtY, unsigned char *prevY)
{
	int *motionHist, type;
	char l[300];

	if (!crtFrame->id)
	{
		crtFrame->copyFrame(prevFrame);
	}
	else if (crtFrame->id - prevFrame->id == 2)
	{
		int cnt = 0;
		for (int i=0;i<g_Image_H;i++)
			for (int j=0; j<g_Image_W; j++)
			{
				crtY[cnt] = crtFrame->Y[i][j];
				prevY[cnt] = prevFrame->Y[i][j];
				cnt++;
			}

		motionHist = motion.GetCameraMotion(prevY,crtY,&type);

		if (crtMotionType == -1)
		{
			crtMotionType = type;
			crtMotionEndFrame = 2;
		}
		else
		{
			if (type == crtMotionType)
				crtMotionEndFrame+=2;
			else
			{
				if (crtMotionEndFrame - crtMotionStartFrame <= 6 || crtMotionType == IRREGULAR)
					IREndFrame = crtMotionStartFrame = crtMotionEndFrame;
				else	// crt motion > 6 then it's a real type of motion and not irregular
				{
					if (IREndFrame - IRStartFrame >= 10 || lastMotionType != crtMotionType)
					{	// write the last and irregular and udate coordinates	
						if (lastMotionEndFrame-lastMotionStartFrame)
						{
							// write the last motion type to file
							sprintf(l,"%d\t%d\t%d\t%d\n",g_videoId,lastMotionType,lastMotionStartFrame,lastMotionEndFrame);
							g_outputFiles[SEFILE]->Write(l,strlen(l));
						}

						// write the irregular motion
						if (IREndFrame - IRStartFrame)
						{
							sprintf(l,"%d\t%d\t%d\t%d\n",g_videoId,IRREGULAR,IRStartFrame,IREndFrame);
							g_outputFiles[SEFILE]->Write(l,strlen(l));
						}
						
						// the last motion becomes the crt one and initialize the irregular
						lastMotionType = crtMotionType;
						lastMotionStartFrame = crtMotionStartFrame;
					}
					// in all cases all counters need to be initiallized
					lastMotionEndFrame = IRStartFrame = IREndFrame = crtMotionStartFrame = crtMotionEndFrame;
				}
				crtMotionEndFrame += 2;
				crtMotionType = type;
			}
		}
		crtFrame->copyFrame(prevFrame);
	}
}
//----------------------------------------------------------------------
void CVideo::calcFeatures_InitCounters()
{
	memset (sCHAvg,0,COLQUANT *4);
	memset (sCHMin,INF,COLQUANT *4);

	memset (sHSVAvg,0,HSV_DIM *4);
	memset (sHSVMin,INF,HSV_DIM *4);

	memset (sCMAvg,0,CM_DIM*4);

	memset (sCLAvg,0,CL_DIM*4);
	memset (sCLMin,INF,CL_DIM *4);

	memset (sTTAvg,0,TT_DIM*4);

	memset (sTEAvg,0,TE_DIM*4);
	memset (sTEMin,INF,TE_DIM *4);
	
	memset (sMHTotal,0,9*4);

	memset (sSCAvg,0,COLQUANT*4);
	
	memset(sHTAvg,0,HT_DIM*4);

	memset(sEHAvg,0,EH_DIM*4);
	
	memset(shotMotionType,0,8*4);
}

//------------------------------------------------------------------------
void CVideo::specialEffects_WriteLastPart()
{
	char l[300];

	crtMotionEndFrame+=2;
	if (crtMotionEndFrame - crtMotionStartFrame <=6 || crtMotionType == IRREGULAR)
	{	// there will not be current motion
		IREndFrame = crtMotionEndFrame;
		if (IREndFrame - IRStartFrame <= 10)
		{	// the whole motion will be like the last motion
			sprintf(l,"%d\t%d\t%d\t%d\n",g_videoId,lastMotionType,lastMotionStartFrame,IREndFrame);
			g_outputFiles[SEFILE]->Write(l,strlen(l));
		}
		else
		{	// write the last and the irregular
			if (lastMotionEndFrame - lastMotionStartFrame)
			{
				sprintf(l,"%d\t%d\t%d\t%d\n",g_videoId,lastMotionType,lastMotionStartFrame,lastMotionEndFrame);
				g_outputFiles[SEFILE]->Write(l,strlen(l));
			}
			if (IREndFrame-IRStartFrame)
			{
				sprintf(l,"%d\t%d\t%d\t%d\n",g_videoId,IRREGULAR,IRStartFrame,IREndFrame);
				g_outputFiles[SEFILE]->Write(l,strlen(l));
			}
		}
	}
	else
	{	// there is a real crt Motion its type is a regular one
		if (IREndFrame - IRStartFrame <= 10 && lastMotionType == crtMotionType)
		{ // write the whole as the last motion type
			sprintf(l,"%d\t%d\t%d\t%d\n",g_videoId,lastMotionType,lastMotionStartFrame,IREndFrame);
			g_outputFiles[SEFILE]->Write(l,strlen(l));
		}
		else
		{	// write 3 types of motion
			if (lastMotionEndFrame - lastMotionStartFrame)
			{
				sprintf(l,"%d\t%d\t%d\t%d\n",g_videoId,lastMotionType,lastMotionStartFrame,lastMotionEndFrame);
				g_outputFiles[SEFILE]->Write(l,strlen(l));
			}
			if (IREndFrame-IRStartFrame)
			{
				sprintf(l,"%d\t%d\t%d\t%d\n",g_videoId,IRREGULAR,IRStartFrame,IREndFrame);
				g_outputFiles[SEFILE]->Write(l,strlen(l));
			}
			if (crtMotionEndFrame - crtMotionStartFrame)
			{
				sprintf(l,"%d\t%d\t%d\t%d\n",g_videoId,crtMotionType,crtMotionStartFrame,crtMotionEndFrame);
				g_outputFiles[SEFILE]->Write(l,strlen(l));
			}
		}

	}

}
