#include "stdafx.h"
#include "stdlib.h"
#include "global.h"
#include "frame.h"
#include "time.h"

#include <math.h>

int getThreshold(int *f, int maxSize,float initialP1)
{
	float P0T,P1T,P0T1,P1T1,H0T,H1T,H0T1,H1T1,maxEntropy;
	int i,T;

	P0T = 1;				// P0(T)
	P1T = initialP1;		// P1(T)
	H0T = 0;				// Hsc(T)
	H1T = 0;				// Hnsc(T)

	for (i=0; i< maxSize ;i++)
		if (f[i])
			H1T += - (f[i]/P1T) * (float) log(f[i]/P1T);

	maxEntropy = 0;
	T = 0;

	for (i=0;i < maxSize;i++)
	{
		if (f[i])
		{
			P0T1 = P0T + f[i];
			P1T1 = P1T - f[i];
			H0T1 = (float) (P0T*H0T - f[i]*log(f[i]/P0T1) - P0T*log(P0T/P0T1))/P0T1;
			H1T1 = (float) (P1T*H1T + f[i]*log(f[i]/P1T1) - P1T*log(P1T/P1T1))/P1T1;
	
			if (H0T1 + H1T1 > maxEntropy)
			{
				maxEntropy = H1T1 + H0T1;
				T = i;
			}

			H0T = H0T1;
			H1T = H1T1;
			P0T = P0T1;
			P1T = P1T1;
		}
	}

	return(T);
}

//================================================
float getAbsolute(float x)
{
	if (x<0) 
		return (-x);
	else
		return (x);
}

//=================================================
int *calculateFrequencies(float * x, int xSize, int start, int fSize, int mapping, float maxValue)
{
	// this function has as an input an array of float values with size xSize
	// we want to calculate 
	// the frequencies of the occurance of those values, the frequencies will be places 
	// in an array and returned to the calling function. fSize is the size of the 
	// frequency array. mapping is a flag 0: calculate exact frequencies, 1: map
	// the values to the size of the array, 2 truncate after the sizer of the array

	// array of frequencies
	int *f,i;
	f = (int *) malloc(sizeof(int) * (fSize+1));	// should be freed from the calling function
	for (i=0; i< fSize + 1;i++) f[i] = 0;

	// caluclate frequencies
	for (i = start; i< xSize; i++)
	{
		if (mapping ==0)	// exact
		{
			f[(int) x[i]]++;
		}
		else if (mapping == 1)	// map
		{
			for(int j=0;j<fSize + 1;j++)
			{
				if((x[i] >= j*maxValue/fSize) && (x[i] < (j+1) * maxValue/fSize))
				{
					f[j]++;
					break;
				}
			}
		}
		else	// truncate
		{
			if ((int) x[i] < fSize)
			{
				f[(int) x[i]]++;
			}
		}
	}

	// return array of frequencies
	return (f);
}
//----------------------------------------------------
int getMin(float *f, int fSize)
{
	/* return the index of the array of the min element in it. f is the array of 
	values and fSize is the size of the array */

	int min=0;
	for(int i= 1;i<fSize;i++)
	{
		if(f[i] <f[min]) min=i;
	}
	return min;
}
//-----------------------------------------------------
COLORREF YUVToRGB(BYTE y, BYTE  u, BYTE v)

{

	float r,g,b;
			
	r = (float) y + (float) v - 128;
	b = (float) y + (float) u - 128;
	g = (float) (((float)y - 0.299*(float)r-0.114*(float)b)/(float) 0.587);

	if (r < 0) r=0; if (r > 255) r=255;
	if (g < 0) g=0;	if (g > 255) g=255;
	if (b < 0) b=0;	if (b > 255) b=255;

		
	return RGB((BYTE) r,(BYTE) g,(BYTE) b);

}
//----------------------------------------------------------

void constructOutputFiles(CString fileName)
{
	/* the given file name path...\data\file.ext
	we will initialize the array of outputFiles so that the output files
	will be path.... \metadata\fileSC.txt, fileSS, fileTS,fileMV,fileKF
	for the output of scenecuts, spatial Seg, temporal seg, motion vec, Key frames
	*/
	if (g_sceneCuts || g_keyFrames)
	{
		g_outputFiles[SCFILE] = new CFile(fileName+"SC.txt", CFile::modeCreate | CFile::modeReadWrite);
		g_outputFiles[KFFILE] = new CFile(fileName+"KF.txt", CFile::modeCreate | CFile::modeWrite);
	}
	else if (g_features || g_specialEffects)	// because the special effect is done in the second path we need to open the SC file as read only even if we will not use it
		g_outputFiles[SCFILE] = new CFile(fileName+"SC.txt", CFile::modeRead);
	
	if (g_spatialSeg)
		g_outputFiles[SSFILE] = new CFile(fileName+"SS.txt",CFile::modeCreate | CFile::modeWrite);

	if (g_temporalSeg)
		g_outputFiles[TSFILE] = new CFile(fileName+"TS.txt",CFile::modeCreate | CFile::modeWrite);

	if (g_MH)
		g_outputFiles[MVFILE] = new CFile(fileName + "MV.txt",CFile::modeCreate | CFile::modeReadWrite);

	if (g_specialEffects)
		g_outputFiles[SEFILE] = new CFile(fileName + "SE.dat",CFile::modeCreate | CFile::modeWrite);

	if (g_CH)
	{
		g_outputFiles[CHFILE] = new CFile(fileName+"CH.txt", CFile::modeCreate | CFile::modeWrite );
		g_outputFiles[CHFFILE] = new CFile(fileName+"CHF.txt", CFile::modeCreate | CFile::modeWrite );
	}
	if (g_CH2)
	{
		g_outputFiles[CHHSVFILE] = new CFile(fileName+"CHHSV.txt", CFile::modeCreate | CFile::modeWrite );
		g_outputFiles[CHHSVFFILE] = new CFile(fileName+"CHHSVF.txt", CFile::modeCreate | CFile::modeWrite );
	}
	if (g_CM)
	{
		g_outputFiles[CMFILE] = new CFile(fileName+"CM.txt", CFile::modeCreate | CFile::modeWrite );
		g_outputFiles[CMFFILE] = new CFile(fileName+"CMF.txt", CFile::modeCreate | CFile::modeWrite );
	}
	if (g_CL)
	{
		g_outputFiles[CLFILE] = new CFile(fileName+"CL.txt", CFile::modeCreate | CFile::modeWrite );
		g_outputFiles[CLFFILE] = new CFile(fileName+"CLF.txt", CFile::modeCreate | CFile::modeWrite );
	}
	if (g_TT)
	{
		g_outputFiles[TTFILE] = new CFile(fileName+"TT.txt", CFile::modeCreate | CFile::modeWrite );
		g_outputFiles[TTFFILE] = new CFile(fileName+"TTF.txt", CFile::modeCreate | CFile::modeWrite );
	}
	if (g_TE)
	{
		g_outputFiles[TEFILE] = new CFile(fileName+"TE.txt", CFile::modeCreate | CFile::modeWrite );
		g_outputFiles[TEFFILE] = new CFile(fileName+"TEF.txt", CFile::modeCreate | CFile::modeWrite );
	}
	if (g_MH)
	{
		g_outputFiles[MHFILE] = new CFile(fileName+"MH.txt", CFile::modeCreate | CFile::modeWrite );
		g_outputFiles[MHFFILE] = new CFile(fileName+"MHF.txt", CFile::modeCreate | CFile::modeWrite );
	}
	if (g_DC)
	{
		g_outputFiles[DCFILE] = new CFile(fileName+"DC.txt", CFile::modeCreate | CFile::modeWrite );
		g_outputFiles[DCFFILE] = new CFile(fileName+"DCF.txt", CFile::modeCreate | CFile::modeWrite );
	}
	if (g_SC)
	{
		g_outputFiles[SCOLFILE] = new CFile(fileName+"SCOL.txt", CFile::modeCreate | CFile::modeWrite );
		g_outputFiles[SCOLFFILE] = new CFile(fileName+"SCOLF.txt", CFile::modeCreate | CFile::modeWrite );
	}

	if (g_HT)
	{
		g_outputFiles[HTFILE] = new CFile(fileName+"HT.txt", CFile::modeCreate | CFile::modeWrite );
		g_outputFiles[HTFFILE] = new CFile(fileName+"HTF.txt", CFile::modeCreate | CFile::modeWrite );
	}

	if (g_EH)
	{
		g_outputFiles[EHFILE] = new CFile(fileName+"EH.txt", CFile::modeCreate | CFile::modeWrite );
		g_outputFiles[EHFFILE] = new CFile(fileName+"EHF.txt", CFile::modeCreate | CFile::modeWrite );
	}

	// the data files of the key frames and video shots
	if (g_sceneCuts || g_keyFrames)
	{
		g_outputFiles[KFDATFILE] = new CFile(fileName+"KF.dat", CFile::modeCreate | CFile::modeWrite);
		g_outputFiles[VSDATFILE] = new CFile(fileName+"VS.dat", CFile::modeCreate | CFile::modeWrite);
		g_outputFiles[VSOFFDATFILE] = new CFile(fileName+"VSOFF.dat", CFile::modeCreate | CFile::modeWrite);
	}
//	g_outputFiles[TEMPFILE] = new CFile(fileName+"temp.txt",CFile::modeCreate | CFile::modeWrite);
}

//-----------------------------------------------------
CString constructFileName(CString originalName, CString addPart)
{
	int dummy = originalName.Replace("\\data\\","\\metadata\\");
	originalName.Delete(originalName.ReverseFind('.'),4);
	return (originalName + addPart);
}


//------------------------------------------------------
void YUV2R_G_B(CFrame *crtFrame,unsigned char *r, unsigned char *g, unsigned char *b)
{
	int cnt = 0;
	for (int i=0; i < g_Image_H; i++)
	{
		for (int j=0; j < g_Image_W; j++)
		{
			b[cnt] = 1.164*(crtFrame->Y[i][j] - 16) + 2.018*(crtFrame->V[i/2][j/2] - 128);
			g[cnt] = 1.164*(crtFrame->Y[i][j] - 16) - 0.813*(crtFrame->U[i/2][j/2] - 128) - 0.391*(crtFrame->V[i/2][j/2] - 128);
			r[cnt] = 1.164*(crtFrame->Y[i][j] - 16) + 1.596*(crtFrame->U[i/2][j/2] - 128);
			cnt++;
		}
	}
}
//--------------------------------------------------------
int getRandomNumber(int min, int max)
{
	// the seed should be initialized in the main program with srand()
	int x;

	if (min == max)
		return (max);

	// get a random number between x and y
	x = rand()%max;
	
	if (x >= min)
	{
		return (x);
	}
	else
	{
		x = x * ((max-min)/min) + min;
	}
	if (!between(x,min,max))
	{
		AfxMessageBox("error calculating random value");
		return ((min+max) / 2);
	}
	return(x) ;
}
//-----------------------------------------------------------
int between(double n, double x, double y)
{
	double min=0,max=0;
	// return 1 if n between x and y and 0 otherwise
	if (x <= y)
	{
		min = x;
		max = y;
	}
	else
	{
		min = y;
		max = x;
	}

	if (n >= min && n<=max)
		return (1);
	else
		return (0);
}

//-----------------------------------------------------------------
void updateShotStatistics(float *avgArr, float *minArr, float *h, int dim)
{
	for (int i=0; i<dim ; i++)
	{
		avgArr[i] += h[i];
		if (h[i] < minArr[i])
			minArr[i] = h[i];
	}
}

//------------------------------------------------------------------
void updateShotStatistics(float *avgArr,float *h, int dim)
{
	for (int i=0; i<dim ; i++)
	{
		avgArr[i] += h[i];
	}
}
//--------------------------------------------------------------------
char * getStringFromArray(float *h, int dim)
{
	char l[10000], temp[50];

	strcpy(l,"");
	for (int i=0;i < dim ; i++)
	{
		if (!h[i])
			strcat(l," 0");
		else
		{
			sprintf(temp," %f",h[i]);
			strcat(l,temp);
		}
	}
	return l;
}